<!DOCTYPE html>
<html lang="en">

  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <meta name="author" content="{{ site.author }}">
    {%- if description %}
    <meta name="description" content="{{ description }}">
    {%- else %}
    <meta name="description" content="{{ site.description }}">
    {%- endif %}
    <title>{% block title %}{% endblock %} | {{ site.description }}</title>
  </head>

  <body>
    <header>
      <h1><a href="/">{{ site.title }}</a></h1>
      <nav>
        <h2>{{ site.description }}</h2>
        <ul>
          <li><a href="/">Blog</a></li>
          <li><a href="/tags/">Tags</a></li>
          <li><a href="/atom.xml">Atom Feed</a></li>
        </ul>
      </nav>
    </header>

    <main>
    {% block content %}
    {% endblock %}
    </main>

  </body>

</html>

