{% extends "base.html" %}

{% block title %}{{ title }}{% endblock %}

{% block content %}

  {% if title %}
  <h2>{{ title }}</h2>
  {% endif %}

  <aside>
    <p>published on {{ date.date() }}

    {% if tags %}
    · tagged with
      {% for tag in tags|sort(case_sensitive=true) %}
        {%- if not loop.first and not loop.last %}, {% endif -%}
        {%- if loop.last and not loop.first %} and {% endif %}
        <a href="/tags/{{ tag }}.html">#{{ tag }}</a>
      {%- endfor %}
    {% endif %}
    </p>
  </aside>


  {{ content }}

{% endblock %}
