{% extends "base.html" %}

{% block title %}{{ site.title }}{% endblock %}

{% block content %}
{% for entry in archive %}

  {% if entry.title %}
  <h1><a href="{{entry.dst}}">{{entry.title}}</a></h1>

  {% if entry.description %}
  <p>— {{ entry.description }}</p>
  {% endif %}

  {% endif %}

  <p>Written on {{ entry.date.date() }}.</p>

{% endfor %}
{% endblock %}
